/*
 *  $Id$
 */

/**
 *  @file btchat.c
 *
 *  @project btChat (Collin's btChat) (http://www.mulliner.org/bluetooth/btchat/)
 *
 *  @author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 *  main protocol handler
 *
 */

#include "internal.h"

#include "account.h"
#include "accountopt.h"
#include "debug.h"
#include "notify.h"
#include "proxy.h"
#include "prpl.h"
#include "server.h"
#include "util.h"
#include "version.h"

#include "btchat_util.h"
#include "btchat_client_protocol.h"
#include "btchat.h"
#include "btchat_backend.h"
#include "btchat_bluetooth_class.h"

static void btchat_set_away(GaimConnection *gc, const char *state, const char *msg)
{
	if (gc->away) {
		g_free(gc->away);
		gc->away = NULL;
	}
}

static GList *btchat_away_states(GaimConnection *gc)
{
	GList *m = NULL;
	
	return(m);
}

static GList *btchat_buddy_menu(GaimConnection *gc, const char *who)
{
	GList *m = NULL;

	return(m);
}

static GList *btchat_node_menu(GaimBlistNode *node)
{
	return(NULL);
}


/**
 *  @brief main protocol handler
 *
 *  @param data   pointer to protocol globals
 *  @param source file descriptor to read
 *  @param cond   input condition (?)
 */
static void main_callback(gpointer data, gint source, GaimInputCondition cond)
{
	GaimConnection *gc = data;
	struct btchat_globals_t *btg = gc->proto_data;
	struct btchat_event_t *e = NULL;
	int res;
	int ret_val;
	
	
	/* check source */
	if (source == 0) {
		gaim_connection_error(gc, _("No connection!"));
		return;
	}
	if (btg->socket != source)	btg->socket = source;

	/* read bytes */
	res = btchat_read(btg);
	if (res != 1) {
		gaim_debug(GAIM_DEBUG_ERROR, "btchat", "main_callback(): btchat_read() failed - CRITICAL!\n");
		gaim_connection_error(gc, _("Unable to read socket!"));
		return;
	}

	/* get all available events until a null_event is found */
	do {
		btchat_event_get(btg, &e);
		res = e->etype;

		if (e == NULL) return;

		switch (e->etype) {
		case BTCHAT_EVENT_NULL:
			/* do nothing */
			break;
		case BTCHAT_EVENT_ACCOUNT:
			ret_val = handle_event_node(gc, e);
			break;
		case BTCHAT_EVENT_MSG:
			ret_val = handle_event_msg_received(gc, e);
			break;
		case BTCHAT_EVENT_NOTIFY:
			break;
		case BTCHAT_EVENT_STATUS:
			break;
		default:
			gaim_debug(GAIM_DEBUG_ERROR, "btchat", "main_callback: unsupported event %d\n", e->etype);
			break;
		}
	
		/* after event is processed free memory */
		btchat_event_free(&e);
		
	} while (res != BTCHAT_EVENT_NULL);
}

void login_callback(gpointer data, gint source, GaimInputCondition cond)
{
	GaimConnection *gc = data;
	struct btchat_globals_t *btg = gc->proto_data;
	

	/* get connection */	
	if (!g_list_find(gaim_connections_get_all(), gc)) {
		if (source >= 0) close(source);
		return;
	}

	/* are we connected? */
	if (source < 0) {
		gaim_connection_error(gc, _("Unable to connect!"));
		return;
	}

	/* save socket */
	btg->socket = source;
	
	if (gc->inpa == 0) {
		gaim_debug(GAIM_DEBUG_MISC, "btchat", "login_callback(): gc->inpa == 0 setting fd watch\n");
		gc->inpa = gaim_input_add(btg->socket, GAIM_INPUT_READ, login_callback, gc);
	}
	
	/* remove login_callback */
	if (gc->inpa) gaim_input_remove(gc->inpa);
		
	/* set main_callback for incoming data */
	gc->inpa = gaim_input_add(btg->socket, GAIM_INPUT_READ, main_callback, gc);
	
	/* we are connected now */
	gaim_connection_set_state(gc, GAIM_CONNECTED);
	serv_finish_login(gc);
	btg->state = BTCHAT_CLIENT_STATE_CONNECTED;

	/* send login */
	handle_event_login(gc);

	gaim_debug(GAIM_DEBUG_INFO, "btchat", "login_callback(): returning from callback\n");
}

static void btchat_keepalive(GaimConnection *gc)
{
	return;
}

static void btchat_login(GaimAccount *account)
{
	GaimConnection *gc = gaim_account_get_connection(account);
	struct btchat_globals_t *btg = gc->proto_data = btchat_globals_new();
	char buf[80];
	
		
	/* two steps to get connected */
	gaim_connection_update_progress(gc, _("Connecting..."), 1, 2);

	gc->inpa = 0;
	btg->state = BTCHAT_CLIENT_STATE_CONNECTING;

	if (gaim_proxy_connect(account, BTCHAT_SERVER_NAME, BTCHAT_SERVER_PORT, login_callback, gc) < 0) {
		g_snprintf(buf, sizeof(buf), _("Connect to %s failed"), BTCHAT_SERVER_NAME);
		gaim_connection_error(gc, buf);
		return;
	}
}

static void btchat_close(GaimConnection *gc)
{
	struct btchat_globals_t *btg = gc->proto_data;

/*	const char *domain = {"btChat\0"};
	GaimGroup *g;
	

	g = gaim_find_group(domain);
	if (g != NULL) {
		g = gaim_group_new(domain);
		gaim_blist_remove_group(g);
	}
*/

	if (gc->inpa) {
		gaim_input_remove(gc->inpa);
	}

	close(btg->socket);
	free(gc->proto_data);
	gc->proto_data = NULL;
}

static int btchat_send_im(GaimConnection *gc, const char *who, const char *msg, GaimConvImFlags flags)
{
	return(handle_event_msg_send(gc, who, msg, flags));
}

static void btchat_add_buddy(GaimConnection *gc, GaimBuddy *who, GaimGroup *group)
{
	return;
}

static void btchat_rem_buddy(GaimConnection *gc, GaimBuddy *who, GaimGroup *group)
{
	return;
}

static void btchat_add_buddies(GaimConnection *gc, GList *buddies, GList *groups)
{
	return;
}

static void btchat_dir_search(GaimConnection *gc, const char *first, const char *middle,
			   const char *last, const char *maiden, const char *city, const char *state,
			   const char *country, const char *email)
{
	gaim_notify_error(gc, NULL,
					  _("Unable to access directory"),
					  _("Gaim was unable to search the Directory "
						"because it was unable to connect to the "
						"directory server. Please try again later."));
}

static void btchat_change_passwd(GaimConnection *gc, const char *old, const char *new)
{
    	gaim_notify_error(gc, NULL,
				  _("Unable to change btchat password"),
				  _("Gaim was unable to change your password "
					"due to an error connecting to the "
					"btchat server. Please try again "
					"later."));
}

static GList *btchat_actions(GaimPlugin *plugin, gpointer context)
{
	GList *m = NULL;

	return(m);
}

static void btchat_get_info(GaimConnection *gc, const char *who)
{
	gaim_notify_error(gc, NULL,
					  _("Unable to access user profile."),
					  _("Gaim was unable to access this user's "
						"profile due to an error connecting to the "
						"directory server. Please try again later."));
}

static const char *btchat_list_icon(GaimAccount *a, GaimBuddy *b)
{
	/* get protocol icon */
	if (b == NULL) {
		return("collins-btchat");
	}
	/* get buddy icon */
	else {
		/*  TODO
		 *  return real buddy icons!
		 *  (just until the real buddy icons are selected here!)
		 */
		return("collins-btchat");
	}
}

static void btchat_list_emblems(GaimBuddy *b, char **se, char **sw, char **nw, char **ne)
{
	struct btchat_buddy_info_t *binfo;


	if (b == NULL) {
		if (se != NULL) *se = "offline";
	}
	else {
		if (b->proto_data == NULL) return;
		binfo = (struct btchat_buddy_info_t*) b->proto_data;

		switch (binfo->node.state) {
		case BTCHAT_NODE_STATE_OFFLINE:
			*se = "offline";
			break;
		case BTCHAT_NODE_STATE_VISIBLE:
			break;
		case BTCHAT_NODE_STATE_ONLINE:
			*se = "freeforchat";
			break;
		case BTCHAT_NODE_STATE_AWAY:
			*se = "away";
			break;
		default:
			break;
		}
	}
}

/**
 *  @brief generate status text shown in buddy list
 *
 *  @param b pointer to buddy
 *
 *  @returns char*
 */
static char* btchat_status_text(GaimBuddy *b)
{
	struct btchat_buddy_info_t *binfo;


	if (b == NULL) {
		return(g_strdup(_("unknown")));
	}
	else {
		if (b->proto_data == NULL) return(g_strdup(_("unknown")));
		binfo = (struct btchat_buddy_info_t*) b->proto_data;
	
		switch (binfo->node.state) {
		case BTCHAT_NODE_STATE_OFFLINE:
			return(g_strdup(_("offline")));
		case BTCHAT_NODE_STATE_VISIBLE:
			return(g_strdup(_("visible")));
		case BTCHAT_NODE_STATE_ONLINE:
			return(g_strdup(_("online")));
		case BTCHAT_NODE_STATE_AWAY:
			return(g_strdup(_("away")));
		default:
			return(g_strdup(_("unknown")));
		}
	}
}

static char* btchat_tooltip_text(GaimBuddy *b)
{
	struct btchat_buddy_info_t *binfo;
	char data[4096];
	char msc[1024];
	char mjc[1024];
	char mic[1024];
	
	
	if (b == NULL) {
		return(g_strdup(_("\n<b>DATA NOT AVAILABLE!</b>")));
	}
	else {
		if (b->proto_data == NULL) return(g_strdup(_("unknown")));
		binfo = (struct btchat_buddy_info_t*) b->proto_data;
		memset(data, 0, sizeof(data));

		/* state */
		strcpy(data, "\n<b>State:</b> ");
		switch (binfo->node.state) {
		case BTCHAT_NODE_STATE_OFFLINE:
			strcat(data, "offline\n");
			break;
		case BTCHAT_NODE_STATE_VISIBLE:
			strcat(data, "visible\n");
			break;
		case BTCHAT_NODE_STATE_ONLINE:
			strcat(data, "online\n");
			break;
		case BTCHAT_NODE_STATE_AWAY:
			strcat(data, "away\n");
			break;
		default:
			strcat(data, "unknown\n");
			break;
		}	
		/* connection type */
		strcat(data, "<b>Connection:</b> ");
		switch (binfo->node.connection) {
		case BTCHAT_CONNECTION_TYPE_NOTCONNECTED:
			strcat(data, "not connected\n");
			break;
		case BTCHAT_CONNECTION_TYPE_INCOMING:
			strcat(data, "direct/incoming\n");
			break;
		case BTCHAT_CONNECTION_TYPE_OUTGOING:
			strcat(data, "direct/outgoing\n");
			break;
		case BTCHAT_CONNECTION_TYPE_NET:
			sprintf(data+strlen(data), "network (distance %d)\n", binfo->node.hops);
			break;
		default:
			strcat(data, "unknown\n");
			break;
		}
		/* discovery method */
		strcat(data, "<b>Discovered by:</b> ");
		switch (binfo->node.discovery_method) {
		case BTCHAT_DISCOVERY_INQUIRY:
			strcat(data, "inquiry scan");
			break;
		case BTCHAT_DISCOVERY_CONNECT:
			strcat(data, "direct connection");
			break;
		case BTCHAT_DISCOVERY_NET:
			strcat(data, "network");
			break;
		default:
			strcat(data, "unknown");
			break;
		}
		/* bluetooth device class */
		if (binfo->node.device_class != 0) {
			strcat(data, "\n<b>DeviceType:</b> ");
			memset(msc, 0, sizeof(msc));
			memset(mjc, 0, sizeof(mjc));
			memset(mic, 0, sizeof(mic));
			bluetooth_classToString(binfo->node.device_class, ", ", msc, mjc, mic);
			/*strcat(data, msc);
			strcat(data, " "); */
			strcat(data, mjc);
			strcat(data, " ");
			strcat(data, mic);
		}
		
		return(g_strdup(data));
	}
}

static void btchat_set_permit_deny_dummy(GaimConnection *gc)
{
	return;
}

static void btchat_permit_deny_dummy(GaimConnection *gc, const char *who)
{
	return;
}

static GaimPlugin *my_protocol = NULL;

static GaimPluginProtocolInfo prpl_info =
{
	OPT_PROTO_NO_PASSWORD,
	NULL,
	NULL,
	NO_BUDDY_ICONS,
	btchat_list_icon,
	btchat_list_emblems,
	btchat_status_text,
	btchat_tooltip_text,
	btchat_away_states,
	btchat_node_menu,
	NULL,
	NULL,
	btchat_login,
	btchat_close,
	btchat_send_im,
	NULL,
	NULL,
	btchat_get_info,
	btchat_set_away,
	NULL,
	btchat_change_passwd,
	btchat_add_buddy,
	btchat_add_buddies,
	btchat_rem_buddy,
	NULL,
	btchat_permit_deny_dummy,
	btchat_permit_deny_dummy,
	btchat_permit_deny_dummy,
	btchat_permit_deny_dummy,
	btchat_set_permit_deny_dummy,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	btchat_keepalive,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

static GaimPluginInfo info =
{
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_PROTOCOL,    /**< type */
	NULL,                    /**< ui_requirement */
	0,                       /**< flags */
	NULL,                    /**< dependencies */
	GAIM_PRIORITY_DEFAULT,   /**< priority */
	
	"prpl-btchat",                                 /**< id */
	"Collin's btChat",                             /**< name */
	VERSION,                                       /**< version */
	N_("Collin's btChat Protocol Plugin"),         /**< summary */
	N_("Collin's btChat Protocol Plugin"),	        /**< description */
	"Collin R. Mulliner <collin@betaversion.net>", /**< author */
	"http://www.mulliner.org/bluetooth/btchat/",   /**< homepage */
	
	NULL,       /**< load */
	NULL,       /**< unload */
	NULL,       /**< destroy */
	NULL,       /**< ui_info */
	&prpl_info, /**< extra_info*/
	NULL,
	NULL
};

static void init_plugin(GaimPlugin *plugin)
{
	my_protocol = plugin;
}

GAIM_INIT_PLUGIN(btchat, init_plugin, info);

/*
 *  $Log$
 */
