/*
 *  $Id$
 */
 
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <termios.h>
#include <time.h>
#include <unistd.h>
#include <assert.h>
#include <string.h>

/**
 *  @brief write n bytes to file descriptor
 *
 *  this is taken from: unix network programming by W.R. Stevens
 * 
 *  @param fd   file descriptor
 *  @param vptr bytes to write out
 *  @param n    number of bytes to write
 *
 *  @returns -1 on failure "n" on success
 */
ssize_t nwrite(int fd, const void *vptr, size_t n)
{
	size_t nleft;
	ssize_t nwritten;
	const char *ptr;
	
	
	ptr = vptr;
	nleft = n;
	while (nleft > 0) {
		if ((nwritten = write(fd, ptr, nleft)) <= 0) {
			if (errno == EINTR) {
				nwritten = 0;
			}
			else {
				return(-1);
			}
		}
		nleft -= nwritten;
		ptr += nwritten;
	}
	return(n);
}

/**
 *  @brief read max n bytes from file descriptor
 *
 *  @param fd   file descriptor
 *  @param vptr store bytes here
 *  @param n    number of bytes to read max.
 *
 *  @returns -1 on failure "n" on success
 */
ssize_t cread(int fd, void *vptr, size_t n)
{
	int result;
	
	
	reread:
	if ((result = read(fd, vptr, n)) < 0) {
		if (result == EINTR) {
			result = 0;
			goto reread;
		}
		else {
			return(-1);
		}
	}
	return(result);
}

/*
 *
 *  TAKEN FROM bluez-libs (www.bluez.org)
 *
 */
 
void baswap(unsigned char *dst, const unsigned char *src)
{
	register int i;
	for (i = 0; i < 6; i++)
		dst[i] = src[5-i];
}

char *batostr(const unsigned char *ba)
{
	char *str = malloc(18);
	if (!str)
		return NULL;

	sprintf(str, "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X",
		ba[0], ba[1], ba[2], 
		ba[3], ba[4], ba[5]);
	return str;
}

unsigned char *strtoba(const char *str)
{
	const char *ptr = str;
	int i;

	__uint8_t *ba = malloc(sizeof(unsigned char)*6);
	if (!ba)
		return NULL;

	for(i = 0; i < 6; i++) {
		ba[i] = (__uint8_t) strtol(ptr, NULL, 16);
		if (i != 5 && !(ptr = strchr(ptr,':')))
			ptr = ":00:00:00:00:00";
		ptr++;
	}
	return ba;
}

int ba2str(const unsigned char *ba, char *str)
{
	__uint8_t b[6];

	baswap(b, ba);
	return sprintf(str, "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X",
		b[0], b[1], b[2], b[3], b[4], b[5]);
}

int str2ba(const char *str, unsigned char *ba)
{
	__uint8_t b[6];
	const char *ptr = str;
	int i;

	for (i = 0; i < 6; i++) {
		b[i] = (__uint8_t) strtol(ptr, NULL, 16);
		if (i != 5 && !(ptr = strchr(ptr, ':')))
			ptr = ":00:00:00:00:00";
		ptr++;
	}
	baswap(ba, b);
	return 0;
}

/*
 *  $Log$
 */
